==> dirac_hashes-0.1.1-py3-none-any.whl: quantum_hash/utils/keys.py <==
"""
Key generation and management functions for quantum-inspired cryptography.
"""

import os
import secrets
import base64
import hashlib
import hmac
from typing import Tuple, Dict, Callable, Any, Optional, Union

# Import the quantum hash functions
from quantum_hash.utils.hash import quantum_hash, _HAVE_OPTIMIZED


def generate_quantum_seed(entropy_bytes: int = 32) -> bytes:
    """
    Generate a high-entropy seed suitable for quantum-inspired key generation.
    
    This combines system entropy with quantum-inspired mixing to create
    a high-quality seed for cryptographic operations.
    
    Args:
        entropy_bytes: Number of bytes of entropy to generate
    
    Returns:
        High-entropy seed bytes
    """
    # Get system entropy
    system_entropy = os.urandom(entropy_bytes)
    
    # Additional entropy source
    extra_entropy = secrets.token_bytes(entropy_bytes)
    
    # Combine entropy sources with XOR
    combined = bytearray(entropy_bytes)
    for i in range(entropy_bytes):
        combined[i] = system_entropy[i] ^ extra_entropy[i]
    
    # Use quantum hash for final mixing
    return quantum_hash(combined, algorithm='improved', digest_size=entropy_bytes)


def generate_keypair(key_size: int = 32, algorithm: str = 'improved',
                    optimized: bool = True) -> Tuple[bytes, bytes]:
    """
    Generate a keypair using quantum-inspired algorithms.
    
    Args:
        key_size: Size of the key in bytes
        algorithm: Algorithm to use for key derivation
        optimized: Whether to use SIMD-optimized implementations when available
    
    Returns:
        Tuple of (private_key, public_key)
    """
    # Generate private key as high-entropy seed
    private_key = generate_quantum_seed(key_size)
    
    # Derive public key from private key
    public_key = quantum_hash(private_key, algorithm=algorithm, 
                             digest_size=key_size, optimized=optimized)
    
    return private_key, public_key


def derive_key(master_key: bytes, purpose: str, key_size: int = 32, 
              algorithm: str = 'improved', optimized: bool = True) -> bytes:
    """
    Derive a subkey from a master key for a specific purpose.
    
    Args:
        master_key: The master key to derive from
        purpose: A string describing the purpose of this key
        key_size: Size of the derived key in bytes
        algorithm: Hash algorithm to use for key derivation
        optimized: Whether to use SIMD-optimized implementations when available
    
    Returns:
        Derived key
    """
    # Convert purpose to bytes if it's a string
    if isinstance(purpose, str):
        purpose = purpose.encode('utf-8')
    
    # Use HKDF-like construction
    # 1. Extract
    salt = b"DiracHash Key Derivation"
    prk = hmac.new(salt, master_key, digestmod=hashlib.sha256).digest()
    
    # 2. Expand using quantum hash
    info = purpose
    derived_key = quantum_hash(prk + info, algorithm=algorithm, 
                              digest_size=key_size, optimized=optimized)
    
    return derived_key


def format_key(key: bytes, format_type: str = 'hex') -> str:
    """
    Format a key for output or storage.
    
    Args:
        key: Key bytes
        format_type: Format type ('hex', 'base64', or 'base58')
    
    Returns:
        Formatted key as a string
    """
    if format_type.lower() == 'hex':
        return key.hex()
    elif format_type.lower() == 'base64':
        return base64.b64encode(key).decode('ascii')
    elif format_type.lower() == 'base58':
        try:
            # Try to import base58 if available
            import base58
            return base58.b58encode(key).decode('ascii')
        except ImportError:
            # Fallback to base64 if base58 is not available
            return base64.b64encode(key).decode('ascii')
    else:
        raise ValueError(f"Unknown format type: {format_type}")


def parse_key(key_str: str, format_type: str = 'hex') -> bytes:
    """
    Parse a formatted key.
    
    Args:
        key_str: Formatted key string
        format_type: Format type ('hex', 'base64', or 'base58')
    
    Returns:
        Key bytes
    """
    if format_type.lower() == 'hex':
        return bytes.fromhex(key_str)
    elif format_type.lower() == 'base64':
        return base64.b64decode(key_str)
    elif format_type.lower() == 'base58':
        try:
            # Try to import base58 if available
            import base58
            return base58.b58decode(key_str)
        except ImportError:
            # Assume it's base64 if base58 is not available
            return base64.b64decode(key_str)
    else:
        raise ValueError(f"Unknown format type: {format_type}")

==> dirac_hashes-0.1.3-py3-none-any.whl: quantum_hash/utils/keys.py <==
"""
Key generation and management functions for quantum-inspired cryptography.
"""

import os
import secrets
import base64
import hashlib
import hmac
from typing import Tuple, Dict, Callable, Any, Optional, Union

# Import the quantum hash functions
from .hash import quantum_hash, _HAVE_OPTIMIZED


def generate_quantum_seed(entropy_bytes: int = 32) -> bytes:
    """
    Generate a high-entropy seed suitable for quantum-inspired key generation.
    
    This combines system entropy with quantum-inspired mixing to create
    a high-quality seed for cryptographic operations.
    
    Args:
        entropy_bytes: Number of bytes of entropy to generate
    
    Returns:
        High-entropy seed bytes
    """
    # Get system entropy
    system_entropy = os.urandom(entropy_bytes)
    
    # Additional entropy source
    extra_entropy = secrets.token_bytes(entropy_bytes)
    
    # Combine entropy sources with XOR
    combined = bytearray(entropy_bytes)
    for i in range(entropy_bytes):
        combined[i] = system_entropy[i] ^ extra_entropy[i]
    
    # Use quantum hash for final mixing
    return quantum_hash(combined, algorithm='improved', digest_size=entropy_bytes)


def generate_keypair(key_size: int = 32, algorithm: str = 'improved',
                    optimized: bool = True) -> Tuple[bytes, bytes]:
    """
    Generate a keypair using quantum-inspired algorithms.
    
    Args:
        key_size: Size of the key in bytes
        algorithm: Algorithm to use for key derivation
        optimized: Whether to use SIMD-optimized implementations when available
    
    Returns:
        Tuple of (private_key, public_key)
    """
    # Generate private key as high-entropy seed
    private_key = generate_quantum_seed(key_size)
    
    # Derive public key from private key
    public_key = quantum_hash(private_key, algorithm=algorithm, 
                             digest_size=key_size, optimized=optimized)
    
    return private_key, public_key


def derive_key(master_key: bytes, purpose: str, key_size: int = 32, 
              algorithm: str = 'improved', optimized: bool = True) -> bytes:
    """
    Derive a subkey from a master key for a specific purpose.
    
    Args:
        master_key: The master key to derive from
        purpose: A string describing the purpose of this key
        key_size: Size of the derived key in bytes
        algorithm: Hash algorithm to use for key derivation
        optimized: Whether to use SIMD-optimized implementations when available
    
    Returns:
        Derived key
    """
    # Convert purpose to bytes if it's a string
    if isinstance(purpose, str):
        purpose = purpose.encode('utf-8')
    
    # Use HKDF-like construction
    # 1. Extract
    salt = b"DiracHash Key Derivation"
    prk = hmac.new(salt, master_key, digestmod=hashlib.sha256).digest()
    
    # 2. Expand using quantum hash
    info = purpose
    derived_key = quantum_hash(prk + info, algorithm=algorithm, 
                              digest_size=key_size, optimized=optimized)
    
    return derived_key


def format_key(key: bytes, format_type: str = 'hex') -> str:
    """
    Format a key for output or storage.
    
    Args:
        key: Key bytes
        format_type: Format type ('hex', 'base64', or 'base58')
    
    Returns:
        Formatted key as a string
    """
    if format_type.lower() == 'hex':
        return key.hex()
    elif format_type.lower() == 'base64':
        return base64.b64encode(key).decode('ascii')
    elif format_type.lower() == 'base58':
        try:
            # Try to import base58 if available
            import base58
            return base58.b58encode(key).decode('ascii')
        except ImportError:
            # Fallback to base64 if base58 is not available
            return base64.b64encode(key).decode('ascii')
    else:
        raise ValueError(f"Unknown format type: {format_type}")


def parse_key(key_str: str, format_type: str = 'hex') -> bytes:
    """
    Parse a formatted key.
    
    Args:
        key_str: Formatted key string
        format_type: Format type ('hex', 'base64', or 'base58')
    
    Returns:
        Key bytes
    """
    if format_type.lower() == 'hex':
        return bytes.fromhex(key_str)
    elif format_type.lower() == 'base64':
        return base64.b64decode(key_str)
    elif format_type.lower() == 'base58':
        try:
            # Try to import base58 if available
            import base58
            return base58.b58decode(key_str)
        except ImportError:
            # Assume it's base64 if base58 is not available
            return base64.b64decode(key_str)
    else:
        raise ValueError(f"Unknown format type: {format_type}")
